--[[
Name: LibResComm-1.0
Revision: $Revision: 70954 $
Author(s): Polleke (polleke@gmail.com)
Documentation: http://www.wowace.com/index.php/LibResComm-1.0
SVN: http://svn.wowace.com/wowace/trunk/LibResComm-1.0
Description: Keeps track of resurrection spells cast in the raid group
Dependencies: LibStub, CallbackHandler-1.0
]]

local MAJOR_VERSION = "LibResComm-1.0"
local MINOR_VERSION = tonumber(("$Revision: 70954 $"):match("%d+")); 
 
local lib = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then return end

local playerName = UnitName('player');
local playerClass = select(2, UnitClass('player'));
local isResser = (playerClass == "PRIEST") or (playerClass == "SHAMAN") or (playerClass == "PALADIN");

-----------------
-- Event Frame --
-----------------

lib.EventFrame = lib.EventFrame or CreateFrame("Frame");
lib.EventFrame:SetScript("OnEvent", function (this, event, ...) lib[event](lib, ...) end)
lib.EventFrame:UnregisterAllEvents();
lib.EventFrame:RegisterEvent("PLAYER_LOGIN");
lib.EventFrame:RegisterEvent("PLAYER_ENTERING_WORLD");

-------------------------------
-- Embed CallbackHandler-1.0 --
-------------------------------

lib.Callbacks = LibStub("CallbackHandler-1.0"):New(lib)

-----------------
-- Static Data --
-----------------

-- Last target name from UNIT_SPELLCAST_SENT
local SentTargetName;

-- Mouse down target
local MouseDownTarget;
local WorldFrameHook

-- Version Information Table
local Versions = {};

-- Battleground/Arena/Group Indicators
local InBattlegroundOrArena;
local InRaidOrParty;

-- For tracking STOP messages
local isCasting = false;

local ResSpells = {
    SHAMAN = "Ancestral Spirit",
    PRIEST = "Resurrection",
    PALADIN = "Redemption"
}

---------------
-- Utilities --
---------------
local function commSend(contents, distribution, target)
    SendAddonMessage("ResComm", contents, distribution or (InBattlegroundOrArena and "BATTLEGROUND" or "RAID"), target);
end

local function unitFullName(unit)
    local name, realm = UnitName(unit);
    if (realm and realm ~= "") then
        return name .. "-" .. realm;
    else
        return name;
    end
end

function lib:GetRaidOrPartyVersions()
    local tab = {};

    if (GetNumRaidMembers() > 0) then
        for i = 1, GetNumRaidMembers() do
            local name = unitFullName('raid' .. i);    
            if (not (name == playerName)) then
                tab[name] = Versions[name] or false;
            end
        end
    elseif (GetNumPartyMembers() > 0) then
        for i = 1, GetNumPartyMembers() do
            local name = unitFullName('party' .. i);
            tab[name] = Versions[name] or false;
        end
    end
    
    tab[playerName] = MINOR_VERSION;

    return tab;
end

function lib:GetGuildVersions()
    local tab = {};

    if (IsInGuild()) then
        GuildRoster();

        for i = 1, GetNumGuildMembers(false) do
            local name = GetGuildRosterInfo(i);
            if (not (name == playerName)) then
                tab[name] = Versions[name] or false;
            end
        end  
    end
    
    tab[playerName] = MINOR_VERSION;

    return tab;
end

function lib:GetUnitVersion(unit)
    if (type(unit) ~= "string") then return end

    local targetName = unitFullName(unit);
    if (targetName == playerName) then return MINOR_VERSION end
    return Versions[targetName] or false;
end

--------------------
-- Event Handlers --
--------------------

function lib:UNIT_SPELLCAST_SENT(unit, spellName, spellRank, targetName)
    if (unit ~= 'player') then return end

    SentTargetName = targetName;
end

function lib:UNIT_SPELLCAST_START(unit, spellName, spellRank)
    if (unit ~= 'player') then return end
    
    if( spellName ~= ResSpells[playerClass]) then return end
    
    isCasting = true;
    
    local target = SentTargetName
    
    if( not SentTargetName or SentTargetName == "Unknown" ) then
        target = MouseDownTarget
    end
    
    local endTime = select(6, UnitCastingInfo(unit)) or (GetTime() + 10)*1000;  
    endTime = endTime / 1000;
    
    self.Callbacks:Fire("ResComm_ResStart", playerName, endTime, target);
    
    commSend(string.format("001%s", target));
    
end

function lib:CHAT_MSG_ADDON(prefix, msg, distribution, sender)
    if (prefix ~= "ResComm") then return end
    if (sender == playerName) then return end
           
    -- Get message type
    local msgtype = tonumber(msg:sub(1, 3));
    if (not msgtype) then return end
       
    if(msgtype == 1) then -- START
        local targetName = msg:sub(4, -1);
        
        local endTime = select(6, UnitCastingInfo(sender)) or (GetTime() + 10)*1000;
        
        endTime = endTime / 1000;
        
        self.Callbacks:Fire("ResComm_ResStart", sender, endTime, targetName);
        
    elseif (msgtype == 2) then
        self.Callbacks:Fire("ResComm_ResStop", sender, false);
    elseif (msgtype == 3) then
        self.Callbacks:Fire("ResComm_ResStop", sender, true);
    elseif (msgtype >= 998) then -- AnnounceVersion
        local version = tonumber(msg:sub(4, -1));
        if (version) then
            Versions[sender] = version;

            if (msgtype == 999) then -- RequestVersion
                if (distribution ~= "BATTLEGROUND") then
                    -- Reply in whisper if possible
                    commSend("998" .. tostring(MINOR_VERSION), "WHISPER", sender);
                else
                    -- Reply to inbound distribution channel
                    commSend("998" .. tostring(MINOR_VERSION), distribution);
                end
            end
        end
    end
end

function lib:UNIT_SPELLCAST_SUCCEEDED(unit, spellName)
    if unit ~= 'player' or not isCasting then return end
    
    self.Callbacks:Fire("ResComm_ResStop", playerName, true);

    commSend("003");
    
    isCasting = false
end

function lib:UNIT_SPELLCAST_STOP(unit, spellName)
    if unit ~= 'player' or not isCasting then return end
        
    commSend("002");
    self.Callbacks:Fire("ResComm_ResStop", playerName, false);

    isCasting = false
end

-- Credits to Ora2
function lib:WorldFrameOnMouseDown(...)
	if GameTooltipTextLeft1:IsVisible() then
		local target = select(3, GameTooltipTextLeft1:GetText():find("^Corpse of (.+)$"))
		if target then	
			MouseDownTarget = target
		end
	end
	WorldFrameHook(...)
end

function lib:PLAYER_LOGIN()
    self.EventFrame:RegisterEvent("CHAT_MSG_ADDON");

    -- For keeping track of versions
    self.EventFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
    self.EventFrame:RegisterEvent("RAID_ROSTER_UPDATE")
    
    InRaidOrParty = (GetNumRaidMembers() > 0) or (GetNumPartyMembers() > 0);
    
    self.EventFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
    self.EventFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
    
    WorldFrameHook = WorldFrame:GetScript("OnMouseDown");
    WorldFrame:HookScript("OnMouseDown", lib.WorldFrameOnMouseDown)
    
    lib:PLAYER_REGEN_ENABLED();
    
    -- Announce and request version in group and in guild at login
    commSend("999" .. tostring(MINOR_VERSION));
	if (IsInGuild()) then
		commSend("999" .. tostring(MINOR_VERSION), "GUILD");
	end
end

function lib:PLAYER_REGEN_ENABLED()
    self.EventFrame:RegisterEvent("UNIT_SPELLCAST_SENT");
    self.EventFrame:RegisterEvent("UNIT_SPELLCAST_START");
    self.EventFrame:RegisterEvent("UNIT_SPELLCAST_STOP");
    self.EventFrame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
end

function lib:PLAYER_REGEN_DISABLED()
    self.EventFrame:UnregisterEvent("UNIT_SPELLCAST_SENT");
    self.EventFrame:UnregisterEvent("UNIT_SPELLCAST_START");
    self.EventFrame:UnregisterEvent("UNIT_SPELLCAST_STOP");
    self.EventFrame:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED");
end

function lib:PLAYER_ENTERING_WORLD()
    local it = select(2, IsInInstance());
    InBattlegroundOrArena = (it == "pvp") or (it == "arena");
end

function lib:PARTY_MEMBERS_CHANGED()
    local wasInRaidOrParty = InRaidOrParty;
    InRaidOrParty = (GetNumRaidMembers() > 0) or (GetNumPartyMembers() > 0);
    
    -- Announce and request version when joining a group
    if (not wasInRaidOrParty and InRaidOrParty) then
        commSend("999" .. tostring(MINOR_VERSION));
    end
end

function lib:RAID_ROSTER_UPDATE()
    self:PARTY_MEMBERS_CHANGED();
end
